/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Marcell Gal cell at x-dsl.hu]                *
***********************************************************

called from eventqueue:
handler2k() is run every 4us * 256 => every 1024 us
The name is historical, it was ment to be called
more frequent than that.

Not linked but included, so inlining works well
*/
   /*!
   * \addtogroup g_event_queues Event Queues
   */
/*@{*/

#ifndef __HANDLER_2K__
#define __HANDLER_2K__
#include "adc.h"
#include "fuelcalc.h"
#include "global.h"
#include "wbo2.h"

#ifdef LED_MATRIX
#include <avr/pgmspace.h>
#include "ve.h"
extern uint8_t PROGMEM charset_r[];
#endif

extern struct time_t rtc;

/*
 * TODO: keep the ADC always running with interrupt
 * (interrupt can be made slimmer given some userspace help)
 * and kill handler2k() altogether.
 * Without event activity an event need NOT be scheduled
 * every 262 msec to update etime.now_upper, 
 * since userspace watches that already
 * hint: readTMR() is called from softelapsed() !
 */

static __inline__ void handler2k(void)
{

#ifdef PS2KEYBOARD
	if (kb_lock) { // someone could propably find a better place for this
		kb_lock--; //  needs a 100+ us delay (but not too long)
		if (kb_lock==0)
			release_kb_clock();
	}
#endif

#ifdef WBO2
// do nothing if the wbo2 sampling is working (ADC interrupt is responsible)
	if(wbo2_sampling.nernst_sample_cnt >= WBO2_SAMPLE_COUNT){ // wbo2 sampling was over...
		uint8_t wbo2_sam = etime.wbo2_sampling;	// cache: late writeback
		if (wbo2_sam) --wbo2_sam;
		// if evaluation was not done, we do traditional sampling: mainloop, hurry up!
		if(wbo2_sam == 0 && wbo2_sampling.nernst_sample_cnt == 0xff) {// nernst sampling needed again
			wbo2_sam = WBO2_SAMPLING+1;
			wbo2_sample_nernst();
		} else { // do more non-wbo2 ADC-ing
			if (read_adc_channel(adc_channel)) {  // sample the other channels
				if (++adc_channel >= ADC_CHANNELS)
					adc_channel=0;
				set_adc_channel(adc_channel);
			}
		}
		etime.wbo2_sampling = wbo2_sam;
	}

#else
    // kickstart the adc once in a while
    if (--rtc.adc_clk == 0) {
      rtc.adc_clk = ADC_PERIOD;
      startADC(); //start adc
    }
#endif

#ifdef LED_MATRIX
#define EMPTY 0xFF
	if(--debug.slow == 0){ 
	uint8_t chr, place, idx, col;
	debug.slow = config.lcd_offs[3];
	// increment the col_cnt and see if we need next char
	place = debug.charcol;
	place ++;	// go to next column
	idx = place >> 3;	// split place to char_idx and column inside that char
	col = place & 7;

	// last_idx = config.lcd_offs[1];	// active chars + space
	if(col >= 5){ // we print empty
		PORTA = EMPTY;
		// and advance to next char
		if(idx >= config.lcd_offs[2]){	// last char over, go back to 0
			place = 0;
		} else {	// more characters available:
			place += 8; // to next character
			place &= 0xF8;	// column 0
		}
	} else {
		if(idx >= config.lcd_offs[1])	// some forced space
			PORTA = EMPTY;
		else {
			// look up the character from the ignition map (hack)
			chr = ign.adv_table[idx];		// no extra config, but conflicts with ign
			// look up the column from character bitmap in flash
#if(EMPTY==0)
			PORTA = PRG_RDB(&charset_r[5*(chr - 'A') + col]);
#else	// inverting:
			PORTA = ~ PRG_RDB(&charset_r[5*(chr - 'A') + col]);
#endif
		}
	}
	
	// set the multiplexer to the right column (implement this to your setup)
	// ...
	
	// latch enable signal reusing the LCD's cable delay config
	// _delay_loop_1(config.lcd_delay >> 4); not needed, since falling edge will latch
	PORTE |= 8;
	debug.charcol = place; // store globally
	// _delay_loop_1(config.lcd_delay & 0xf);
	debug.charcol = place;
	PORTE &= ~8;	// 74573 latches the signal
	}
#endif

#ifdef BENCHMARK
	benchstats.handler2k ++;
#endif

}
#endif

/*@}*/

