/*********************************************************\
*  AVR MegaSquirt - 2003                                  *
*  (C) 2003 [Michael Kristensen, mik@caffrey.dk]          *
***********************************************************
*  -- History --                                          *
*    release 0.1 - 06-03-2003                             *
*            http://caffrey.dk/megasquirt                 *
\*********************************************************/

#ifndef IAC_H
#define IAC_H

extern struct iac_t iac;
extern uint8_t idle_rpm;  // the target_rpm


//#define SIMPLE_IAC // this requires changes in config_t
#define PID_IAC

#ifdef SIMPLE_IAC
// defines for config.iac[]
#define RESYNC            0   // set to 0 to disable resync. Any other number is in 'FINE_PERIOD' resolution
#define COARSE_DEV        1   // [rpm/10], ex: 23 <=> 230 rpm deviation
#define ADJUST_COARSE     2   // number of steps moved at each adjustment, packed:
                              // Upper four bits for increasing of rpm, lower 4 bits for decreasing rpm
#define COARSE_PERIOD     3   // 1: 100ms, 2: 200ms, 3: 300ms, etc.
#define FINE_DEV          4   // [rpm/10], ex: 23 <=> 230 rpm deviation
#define ADJUST_FINE       5   // number of steps moved at each adjustment, packed:
                              // Upper four bits for increasing of rpm, lower 4 bits for decreasing rpm
#define FINE_PERIOD       6   // 1: 100ms, 2: 200ms, 3: 300ms, etc.

/*
Suggested configuration:
step_seq : depends on how the stepper is wired up, read the docs
conf : no power off, speed depends on the stepper
max_steps : 100
resync : 15 (every 15 seconds)
tps_thres : depends on throttle pot
fast_idle_temp : 60C degress
cold_rpm : 100 (1000 rpm)
warm_rpm : 85 (850 rpm)
cold_start_pos : 50
warm_start_pos : 20
decel_open : 0
coarse_dev : 20 (+/- 200 rpm)
adjust_coarse : 0x43
coarse_period : 1
fine_dev : 3 (+/- 30 rpm)
adjust_fine : 0x21
fine_period : 10 (every second)
*/

#endif



/*
#ifdef PID_IAC
// defines for config.iac[]
#define START_RPM         0   // right after cranking, this (rpm/10) is added to target_rpm such that the idle rpm temporarily is raised
#define START_COUNT       1   // the extra START_RPM is decreased in START_COUNT [sec/10]
#define PID_CONF          2   // parameters for the pid controller (asymmetric)
#define KP                3   // proportional multiplier
#define KI                4   // integral multiplier
#define KD                5   // differential multiplier
#define DEADBAND          6   // no movement of iac when |current-target_rpm| < deadband. [rpm/10]
#define INTEGRAL_LIMIT    7   // limit the integrator
#define TAU_INCREASE_RPM  8   // timedelay increasing rpm when engine is spinning too slow (keep this small)
#define TAU_DECREASE_RPM  9   // timedelay decreasing rpm when engine is spinning too fast

#endif
*/

/***********************************************************************/
// config.iac_conf bits:
#define power_off_iac  0    // power off stepper 1 step cycle after each move
#define low_power_halt 1    // don't move iac if batt < 8V
#define stepper_iac		2    // enable stepper based iac
#define precise_idle	3    // 0: on/off 1: pwm/stepper based
#define advance_control 4   // 0: off 1: enabled
// use 0 bitvalue for special function (when using v2 IDL output for WOT switch) 

// Do we want independent bit to disable the stepper? grep stepperdisable
// It does not hurt to have stepper and softpwm output at the same time


//bit 7-4 defines speed:
// 0 (fastest, 0ms delay between steps)
// f (slowest, 15ms delay between steps)

/***********************************************************************/
//iac_pid_conf
#define pid_asymmetric 0    // assymetric error
#define tf_disable     1    // disable throttle follower
#define mult           2    // Cell's mult

/***********************************************************************/

#define busy         0
#define direction    1  // 1 == open, 0 == close
#define sync         2  // moving to fully closed position
#define iac_batt_low 3


struct iac_t {
  uint8_t status;        // uses the defines above
  uint8_t count;         // how many steps are requested
  uint8_t position;      // current position

// integral is the only "static" variable, the others are just used for lcd
	int16_t integral;
	int16_t error;
	int16_t rpmdot;

// intermediate results, used for lcd
	int16_t pid;
	int16_t pid_p;
	int16_t pid_i;
	int16_t pid_d;

	uint8_t resync;

	int16_t adv_adjust;
};


// the number of steps besides config.iac[MAX_STEPS], to close the iac at powerup
#define IAC_SYNC 10
#define INTEGRAL_LIMIT_SCALE 7

// hysteresis for fastidle
#define FAST_IDLE_THRES 3

void calc_iac_pos(void);
void init_iac(void);
void set_iac_pos(uint8_t pos);
uint8_t next_step(uint8_t phase);
void move_iac(void);
// void iac_pwm_soft(void);
// void fast_idle_valve(void);
void idle_solenoid(void);
uint8_t idle_advance_adjust(uint8_t adv);

#endif
