
#ifndef KNOCK_H
#define KNOCK_H

#include <inttypes.h>

#define KNOCK_CHAN 8

struct knock_t {
	uint16_t data;  // integration value read from knock IC
	uint8_t channel; // the knock channel which the data came from,
			 // note that 'channel' is updated at the same
			 // time as 'data' is written.
	uint8_t igncount; // the ignition channel for which the data origins

	uint16_t window_start;   // used by dispatcher
	uint16_t window_length;  // used by dispatcher

	uint8_t status;
};

// knock.status defines:
#define KNOCK_IN_ADV_MODE     0
#define KNOCK_NOT_RESPONDING  1
#define knock_window_valid    2

// upper 4 bits of knock.status are state variables
// note: the order must be preserved
#define KNOCK_IDLE          (0)
#define KNOCK_WINDOW_START  (KNOCK_IDLE+1)
#define KNOCK_WINDOW_DONE   (KNOCK_WINDOW_START+1)
#define KNOCK_WINDOW_PICKUP (KNOCK_WINDOW_DONE+1)
#define NOISE_WINDOW_START  (KNOCK_WINDOW_PICKUP+1)
#define NOISE_WINDOW_DONE   (NOISE_WINDOW_START+1)
#define NOISE_WINDOW_PICKUP (NOISE_WINDOW_DONE+1)

#define knock_nextstate() knock.status += 0x10
#define knock_set_state(x) knock.status = (knock.status & 0x0f) | (x<<4)
#define knock_state() (knock.status >> 4)
#define knock_ready() (knock.status & _BV(knock_window_valid))


extern struct knock_t knock;


// this data structure is a bit overkill!
struct knock_cyl_t {
	uint16_t knock_val;
	uint16_t noise_val;
	uint8_t knock_adjust;
	uint8_t kthres_high_cnt;  // duration of each state (unit: knock samples)
	uint8_t kthres_low_cnt;  // duration of each state (unit: knock samples)
	uint8_t state; // the state variable
	uint8_t cnt;  // how long time [15ms] to stay in a state
};

//state definitions for 'state' and 'knock_intensity':
#define K_NO_KNOCK 0
#define K_KNOCK    1

//Why dont we have a define specifying the number of ignition channels?
extern struct knock_cyl_t knock_cyl[KNOCK_CHAN];


struct knock_stats_t {
	uint16_t count;
	uint16_t pos_intensity_cnt; // DEBUG, delete me later on
	uint16_t neg_intensity_cnt; // DEBUG, delete me later on
	uint16_t data;

	uint8_t ve_igncount; // remember which knock_cyl[] the trig2spark belongs to
	uint8_t dwell_err;
	uint8_t adv_err;
};

extern struct knock_stats_t knock_stats;


/*
// eventually these will hit the config_t
// they can probably be merged into a couple of variables
#define RETARD_SMALL_STEP    4
#define RETARD_LARGE_STEP   16

#define ADVANCE_SMALL_STEP   4
//#define ADVANCE_LARGE_STEP   4

#define KNOCK_THRESHOLD  230

#define RETARD_LARGE_DELAY   33  // RETARD_LARGE_STEP degrees / (rate*[15ms unit])
#define RETARD_SMALL_DELAY   16  // [15ms unit]
#define RECOVERY_SMALL_DELAY 16  // [15ms unit]
*/

void handle_knock(void);
void calc_knock_window(void);
uint8_t next_knock_channel(void);
void reset_knock_adjust(void);
void decrease_knock_cnt(void);
void init_knock(void);

#define inc_var8(x) if (x < 0xff) x++


#endif
