

#ifndef SPI_H
#define SPI_H

#ifdef GENBOARDv3

#include <inttypes.h>

// SPI defines...

//*** ADC ***//

#define MCP3208_CHANNELS 8
extern uint16_t mcp3208_data[MCP3208_CHANNELS];

// 04 is the start bit (as on datasheet page 18)
// 02 means single-ended (not differential) input. 
// MCP3208 allows this to be shifted, 
// but the channel selector and the result scale must match
#define MCP3208_CMD_0 0x06
// these are not used, but hardwired:
#define MCP3208_CMD_1 0x00  // this is almost don't care
#define MCP3208_CMD_2 0x00  // this is don't care


//*** KNOCK ***//

#define KNOCK_CHANNELS 2
extern uint16_t knock_data[KNOCK_CHANNELS];

#define KNOCK_CHANNEL_1 0
#define KNOCK_CHANNEL_2 1


#define KNOCK_SET_ADV_MODE   0x71
#define KNOCK_SET_PRESCALER  0x40  // + D[4:0]
#define KNOCK_SET_CHANNEL    0xE0  // + D[0]
#define KNOCK_SET_BANDPASS   0x00  // + D[5:0]
#define KNOCK_SET_GAIN       0x80  // + D[5:0]
#define KNOCK_SET_INTEGRATOR 0xC0  // + D[4:0]

#ifndef KNOCK_OSC
#define KNOCK_OSC (4)
#endif

#if KNOCK_OSC == 4
#define KNOCK_OSC_FREQ       0x00  // 4 MHz
#elif KNOCK_OSC == 16
#define KNOCK_OSC_FREQ       0x06  // 16 MHz
#else
#error "Bad KNOCK oscillator frequency specified"
#endif

//*** SPI ***//

#define NO_CMD      0
#define ADC_READ    1  // upper 4 bits used for selecting channel
#define KNOCK_READ  2
#define KNOCK_SETUP 3  // upper 4 bits used for selecting channel

/*
#define SPI_QUEUE_LENGTH 4

struct spi_queue_t {
	uint8_t q[SPI_QUEUE_LENGTH];
	uint8_t head, tail;
};
*/

struct spi_t {
	uint8_t cnt;
	uint8_t cmd;
	uint8_t tmp_data;

	uint8_t (*next_byte)(uint8_t t);
};


extern struct spi_t spi;


void init_spi(void);
void init_spi_mcp3208(void);
void init_spi_knock(void);
void read_mcp3208(uint8_t chan);
void set_knock_channel(uint8_t chan);
void read_knock(void);
uint8_t spi_knock_busy(void);

void next_spi_cmd(void);


#endif

#endif
