#ifndef __WBO2_H__
#define __WBO2_H__

#include "softpwm.h"
#include "adc.h"
/*
GENBOARDv2 pin definitions for WBO2

Nernst pulsing : PB7 (OC2)
Nernst analog  : ADC7
Pump           : PE3 (OC3A)
Heater         : PE4 (OC3B)

GENBOARDv3 pin definitions for WBO2 #1

Nernst pulsing : PB7 (OC2)
Nernst analog  : ADC1 (EGO_1)
Pump           : PE3 (OC3A)
Heater         : PG0 (HEAT1)

GENBOARDv3 pin definitions for WBO2 #2

Nernst pulsing : PB7 (OC2)
Nernst analog  : ADC7 (EGO_2)
Pump           : PE4 (OC3B)
Heater         : PG1 (HEAT2)

*/

/*
#ifndef PUMP_PORT
#define PUMP 3
#define PUMP_PORT PORTE
#endif
*/

#ifdef GENBOARDv3
//currently only one wbo2 channel

#define HEATER HEAT1
#define HEATER_PORT PORTG
#define NERNST EGO_1

#else

#ifndef HEATER_PORT
#define HEATER 4
#define HEATER_PORT PORTE
#define NERNST 7
#endif

#endif

#define WBO2_SAMPLE_COUNT 17
struct wbo2_sampling_t {
	// this structure is related to raw nernst sampling 
	// shared between all WBO2 channels
	
	uint16_t nernst_sample[WBO2_SAMPLE_COUNT];  // 10bit result from adc
	uint8_t nernst_edgetime[WBO2_SAMPLE_COUNT];  // the time of above samples
	uint8_t nernst_sample_cnt; 
	// 0..WBO2_SAMPLE_COUNT-1: wbo2 sampling
	// WBO2_SAMPLE_COUNT: wbo2 sampling over
	// ff: wbo2 sampling evaluated
};

struct wbo2_t {
	uint8_t status;
	uint8_t Ip_output;	// 0..255 as 0..5V in http://www.diy-wb.com/v_out.htm
// we store the desired heater output power in the softpwm structure
	uint16_t Ri;			// calculated AC: internal resistance of the nernst [8 * (seen jump on ADC+small offset)]
	uint16_t nernst_dc;		// measured DC
	uint16_t started;   // time of starting (used for warmup)
	uint8_t pump_pw;   // pulsewidth of 'pump' low-signal [min..max]
	uint8_t heater_start_dc;	// starting V precalculated from config and coolant
	uint16_t heater_railed_since;	// time the heater started to rail (or begin of fallback state)

	uint8_t heat;  // used for lcd
	uint8_t heater_prescale; // #### find a better name for this var ####
	uint8_t pump_prescale; // #### find a better name for this var ####

// 16-bits enough for the following two variables
	uint16_t Ri_sum;
	uint16_t nernst_dc_sum; 

	// variables used for calulation of AFR
	uint16_t pump_dc_sum;
	uint16_t pump_dc_average;
	uint8_t afr_samples;
	uint16_t current_afr; // ffff means unknown
	uint8_t afr_confidence;  // ff: max confidence, 00: don't use afr value for anything
	uint16_t filtered_afr; // ffff means unknown
	uint16_t current_o2;  // 0 means unknown
};

#define WBO2_HEATER_PRESCALE 8  // Beware when changing! Ri_sum might overflow
#define WBO2_PUMP_PRESCALE 4  // Beware when changing! nernst_dc_sum might overflow

// this specifies how often the AFR is calculated 
// (but 64..160 msec is too rare, earlier 256 msec was hell!):
// period: handler2k_period * wbo2_sampling * wbo2_pump_prescale * wbo2_afr_samples
#define WBO2_AFR_SAMPLES  4  // As usual: Beware when changing!

//status fields:
#define ACTIVE         0
#define IN_WARMUP      1
#define PUMP_ENABLE    2
// #define NERNST_SAMPLING 2
// #define NERNST_READY    3
#define FALLBACK_ACTIVE	(4)


// number of handler2k executions between starting nernst sampling
// TODO: should be more frequent than 16?
#define WBO2_SAMPLING 10

// afr filtering...
#define AFR_FILTER_LENGTH 4

        // 5: 3(8 samples) + 2 bit ADC resolution  [from nernst_acdc()]
        // 2(sum of 4 samples) [PUMP_PRESCALE]
        // -1 (because pid15(dcsum>>1))
        // so total 6 bit
#define WBO2_PUMP_PID_SCALE 6
#define wbo2_pump_pid_set_target(ppd, nndt) pid_set_target(ppd, (uint16_t)nndt << WBO2_PUMP_PID_SCALE);


extern struct wbo2_t wbo2;
extern struct wbo2_sampling_t wbo2_sampling;
extern struct pid_t heater_pid;
extern struct pid_t pump_pid;
extern struct softpwm_t wbo2_heater_pwm;

//void wbo2_pump_low(void);
//void wbo2_pump_high(void);
void wbo2_heater(void);
void init_wbo2(void);
void wbo2_sample_nernst(void);
void nernst_acdc(void);

#endif
